@@ -404,6 +404,12 @@ static int writev_ctx_t__(ei_socket_callbacks *cbs, void *ctx,
     do {
         error = cbs->writev(ctx, (const void *) iov, iovcnt, len, ms);
     } while (error == EINTR);
+
+    /* It should not be possible for writev_ctx_t__ to return EAGAIN,
+       because when the fd is set in non-blocking we always do a select on
+       the fd before trying to write. And if we do not do the select the fd
+       is always in blocking mode. */
+
     return error;
 }
 
@@ -417,7 +423,8 @@ int ei_writev_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx,
     struct iovec *current_iov;
     int current_iovcnt;
     int fd, error;
-    int basic;
+    int non_blocking = !(cbs->flags & EI_SCLBK_FLG_FULL_IMPL) &&
+        ms != EI_SCLBK_INF_TMO;
 
     if (!cbs->writev)
         return ENOTSUP;
@@ -426,12 +433,10 @@ int ei_writev_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx,
     if (error)
         return error;
 
-    basic = !(cbs->flags & EI_SCLBK_FLG_FULL_IMPL);
-    
     for (sum = 0, i = 0; i < iovcnt; ++i) {
 	sum += iov[i].iov_len;
     }
-    if (basic && ms != 0U) {
+    if (non_blocking) {
 	SET_NONBLOCKING(fd);
     } 
     current_iovcnt = iovcnt;
@@ -442,7 +447,7 @@ int ei_writev_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx,
 	error = writev_ctx_t__(cbs, ctx, current_iov, current_iovcnt, &i, ms);
         if (error) {
             *len = done;
-	    if (ms != 0U) {
+	    if (non_blocking) {
 		SET_BLOCKING(fd);
 	    }    
 	    if (iov_base != NULL) {
@@ -478,7 +483,7 @@ int ei_writev_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx,
 	    break;
 	}
     } 
-    if (basic && ms != 0U) {
+    if (non_blocking) {
 	SET_BLOCKING(fd);
     }
     if (iov_base != NULL) {
@@ -752,9 +757,10 @@ int ei_read_fill_ctx__(ei_socket_callbacks *cbs, void *ctx, char* buf, ssize_t *
 int ei_write_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx, const char *buf, ssize_t *len, unsigned ms)
 {
     ssize_t tot = *len, done = 0;
-    int error, fd = -1, basic = !(cbs->flags & EI_SCLBK_FLG_FULL_IMPL);
+    int error, fd = -1, non_blocking = !(cbs->flags & EI_SCLBK_FLG_FULL_IMPL) &&
+        ms != EI_SCLBK_INF_TMO;
     
-    if (basic && ms != 0U) {
+    if (non_blocking) {
         error = EI_GET_FD__(cbs, ctx, &fd);
         if (error)
             return error;
@@ -765,14 +771,14 @@ int ei_write_fill_ctx_t__(ei_socket_callbacks *cbs, void *ctx, const char *buf,
 	error = write_ctx_t__(cbs, ctx, buf+done, &write_len, ms);
         if (error) {
             *len = done;
-	    if (basic && ms != 0U) {
+	    if (non_blocking) {
 		SET_BLOCKING(fd);
 	    }    
 	    return error;
 	}
 	done += write_len;
     } while (done < tot);
-    if (basic && ms != 0U) {
+    if (non_blocking) {
 	SET_BLOCKING(fd);
     }
     *len = done;
