@@ -247,7 +247,7 @@ port_info(_Result, _Item) ->
     erlang:nif_error(undefined).
 
 -spec request_system_task(Pid, Prio, Request) -> 'ok' when
-      Prio :: 'max' | 'high' | 'normal' | 'low',
+      Prio :: 'max' | 'high' | 'normal' | 'low' | 'inherit',
       Type :: 'major' | 'minor',
       Request :: {'garbage_collect', term(), Type}
 	       | {'check_process_code', term(), module()}
@@ -289,10 +289,8 @@ check_process_code(Pid, Module, OptionList)  ->
     Async = get_cpc_opts(OptionList, sync),
     case Async of
 	{async, ReqId} ->
-	    {priority, Prio} = erlang:process_info(erlang:self(),
-						   priority),
 	    erts_internal:request_system_task(Pid,
-					      Prio,
+					      inherit,
 					      {check_process_code,
 					       ReqId,
 					       Module}),
@@ -302,11 +300,9 @@ check_process_code(Pid, Module, OptionList)  ->
 		true ->
 		    erts_internal:check_process_code(Module);
 		false ->
-		    {priority, Prio} = erlang:process_info(erlang:self(),
-							   priority),
 		    ReqId = erlang:make_ref(),
 		    erts_internal:request_system_task(Pid,
-						      Prio,
+						      inherit,
 						      {check_process_code,
 						       ReqId,
 						       Module}),
